#ifndef __IMUMPU6886_H__
#define __IMUMPU6886_H__

#include <stdint.h>
#include <math.h>
#include "rtwtypes.h"
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "nvs_flash.h"
#include "nvs.h"
#include "waijung2_hwdrvlib.h"

#define MPU6886_ADDRESS          0x68
#define MPU6886_WHOAMI           0x75
#define MPU6886_ACCEL_INTEL_CTRL 0x69
#define MPU6886_SMPLRT_DIV       0x19
#define MPU6886_INT_PIN_CFG      0x37
#define MPU6886_INT_ENABLE       0x38
#define MPU6886_ACCEL_XOUT_H     0x3B
#define MPU6886_ACCEL_XOUT_L     0x3C
#define MPU6886_ACCEL_YOUT_H     0x3D
#define MPU6886_ACCEL_YOUT_L     0x3E
#define MPU6886_ACCEL_ZOUT_H     0x3F
#define MPU6886_ACCEL_ZOUT_L     0x40

#define MPU6886_TEMP_OUT_H 0x41
#define MPU6886_TEMP_OUT_L 0x42

#define MPU6886_GYRO_XOUT_H 0x43
#define MPU6886_GYRO_XOUT_L 0x44
#define MPU6886_GYRO_YOUT_H 0x45
#define MPU6886_GYRO_YOUT_L 0x46
#define MPU6886_GYRO_ZOUT_H 0x47
#define MPU6886_GYRO_ZOUT_L 0x48

#define MPU6886_USER_CTRL     0x6A
#define MPU6886_PWR_MGMT_1    0x6B
#define MPU6886_PWR_MGMT_2    0x6C
#define MPU6886_CONFIG        0x1A
#define MPU6886_GYRO_CONFIG   0x1B
#define MPU6886_ACCEL_CONFIG  0x1C
#define MPU6886_ACCEL_CONFIG2 0x1D
#define MPU6886_FIFO_EN       0x23

#define MPU6886_FIFO_ENABLE 0x23
#define MPU6886_FIFO_COUNT  0x72
#define MPU6886_FIFO_R_W    0x74
#define MPU6886_GYRO_OFFSET 0x13
#define ACC_8G_RES (0.000244140625f)
#define GYRO_2000DPS_RES (0.06103515625f)
#define DEG_TO_RAD (0.017453292519943295769236907684886)
typedef union {
  uint8_t raw[14];
  struct {
    uint8_t acc_x_h;
    uint8_t acc_x_l;
    uint8_t acc_y_h;
    uint8_t acc_y_l;
    uint8_t acc_z_h;
    uint8_t acc_z_l;
    uint8_t temp_h;
    uint8_t temp_l;
    uint8_t gyro_x_h;
    uint8_t gyro_x_l;
    uint8_t gyro_y_h;
    uint8_t gyro_y_l;
    uint8_t gyro_z_h;
    uint8_t gyro_z_l;
  } value;
} ImuData_t;
void I2C_Read_NBytes(uint8_t driver_Addr, uint8_t start_Addr, uint8_t number_Bytes, uint8_t* read_Buffer);
void I2C_Write_NBytes(uint8_t driver_Addr, uint8_t start_Addr, uint8_t number_Bytes, uint8_t* write_Buffer);
void setFIFOEnable(_Bool enableflag);
void RestFIFO();
uint16_t ReadFIFOCount();
void ImuUpdateTask(void *arg);
void MadgwickAHRSupdateIMU(float gx, float gy, float gz, float ax, float ay, float az, float *pitch, float *roll, float *yaw);
void MadgwickAHRSetBeta(float beta);
void getAngle(real32_T* balaangle);
void ImuTaskStart(int16_t x_offset, int16_t y_offset, int16_t z_offset, SemaphoreHandle_t* i2c_lock);
void Imuinit();
void calibrationInit();
void calibrationGet(int16_t *gyro_x_offset, int16_t *gyro_y_offset,
                      int16_t *gyro_z_offset, float *angle_center);

#endif